<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Entity;

use Sylius\Component\Core\Model\ProductVariantInterface;
use Sylius\Component\Product\Model\ProductInterface;

class QuoteItem implements QuoteItemInterface
{
    protected ?int $id;

    protected ?QuoteInterface $quote = null;

    protected ?ProductVariantInterface $variant = null;

    protected ?int $quantity = 0;

    protected ?int $unitPrice = 0;

    protected ?int $total = 0;

    protected ?string $productName = null;

    protected ?string $variantName = null;

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getQuote(): ?QuoteInterface
    {
        return $this->quote;
    }

    public function setQuote(?QuoteInterface $quote): self
    {
        $this->quote = $quote;

        return $this;
    }

    public function getVariant(): ?ProductVariantInterface
    {
        return $this->variant;
    }

    public function setVariant(?ProductVariantInterface $variant): self
    {
        $this->variant = $variant;

        return $this;
    }

    public function getQuantity(): ?int
    {
        return $this->quantity;
    }

    public function setQuantity(?int $quantity): self
    {
        $this->quantity = $quantity;

        $this->recalculateTotal();

        return $this;
    }

    public function getUnitPrice(): ?int
    {
        return $this->unitPrice;
    }

    public function setUnitPrice(?int $unitPrice): self
    {
        $this->unitPrice = $unitPrice;

        $this->recalculateTotal();

        return $this;
    }

    protected function recalculateTotal(): void
    {
        $this->total = $this->unitPrice * $this->quantity;

        if (null !== $this->quote) {
            $this->quote->recalculateTotal();
        }
    }

    public function getTotal(): ?int
    {
        return $this->total;
    }

    public function setTotal(?int $total): self
    {
        $this->total = $total;

        return $this;
    }

    public function getProductName(): ?string
    {
        return $this->productName;
    }

    public function setProductName(?string $productName): self
    {
        $this->productName = $productName;

        return $this;
    }

    public function getVariantName(): ?string
    {
        return $this->variantName;
    }

    public function setVariantName(?string $variantName): self
    {
        $this->variantName = $variantName;

        return $this;
    }

    public function getProduct(): ProductInterface
    {
        return $this->variant->getProduct();
    }

    public function equals(QuoteItemInterface $item): bool
    {
        return $this === $item || ($item->getVariant() === $this->variant);
    }
}
