<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\EventListener\Sales;

use Dedi\SyliusQuotePlugin\Specification\IsOrderItemFromQuoteSpecification;
use InvalidArgumentException;
use Sylius\Bundle\ResourceBundle\Event\ResourceControllerEvent;
use Sylius\Component\Core\Model\OrderItemInterface;

class OrderItemPreDeleteListener
{
    private readonly IsOrderItemFromQuoteSpecification $isOrderItemFromQuote;

    public function __construct(IsOrderItemFromQuoteSpecification $isOrderItemFromQuote)
    {
        $this->isOrderItemFromQuote = $isOrderItemFromQuote;
    }

    public function __invoke(ResourceControllerEvent $event): void
    {
        /** @var OrderItemInterface $orderItem */
        $orderItem = $event->getSubject();

        if (!$orderItem instanceof OrderItemInterface) {
            throw new InvalidArgumentException(sprintf(
                'Expected an instance of "%s", got "%s"',
                OrderItemInterface::class,
                get_class($orderItem),
            ));
        }

        if ($this->isOrderItemFromQuote->__invoke($orderItem)) {
            $event->stop('dedi_quote.cart_item.cannot_update_from_quote');
        }
    }
}
