<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\EventListener\Sales;

use Dedi\SyliusQuotePlugin\Context\QuoteContextInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Exception\QuoteNotFoundException;
use Dedi\SyliusQuotePlugin\Storage\QuoteStorageInterface;
use InvalidArgumentException;
use Symfony\Component\HttpKernel\Event\ResponseEvent;

class SessionQuoteEventListener
{
    private readonly QuoteContextInterface $quoteContext;

    private readonly QuoteStorageInterface $quoteStorage;

    public function __construct(QuoteContextInterface $quoteContext, QuoteStorageInterface $quoteStorage)
    {
        $this->quoteContext = $quoteContext;
        $this->quoteStorage = $quoteStorage;
    }

    public function __invoke(ResponseEvent $event)
    {
        if (!$event->isMainRequest()) {
            return;
        }

        $request = $event->getRequest();

        if (!$request->hasSession() || !$request->getSession()->isStarted()) {
            return;
        }

        try {
            $quote = $this->quoteContext->getQuote();
            if (!$quote instanceof QuoteInterface) {
                throw new InvalidArgumentException(sprintf(
                    'Expected an instance of "%s", got "%s"',
                    QuoteInterface::class,
                    get_class($quote),
                ));
            }
        } catch (QuoteNotFoundException $exception) {
            return;
        }

        if (null !== $quote->getId() && null !== $quote->getChannel()) {
            $this->quoteStorage->setForChannel($quote->getChannel(), $quote);
        }
    }
}
