<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Factory;

use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Sylius\Component\Product\Model\ProductInterface;
use Sylius\Component\Product\Resolver\ProductVariantResolverInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;

class QuoteItemFactory implements QuoteItemFactoryInterface
{
    private readonly FactoryInterface $decoratedFactory;

    private readonly ProductVariantResolverInterface $productVariantResolver;

    public function __construct(
        FactoryInterface $decoratedFactory,
        ProductVariantResolverInterface $productVariantResolver,
    ) {
        $this->decoratedFactory = $decoratedFactory;
        $this->productVariantResolver = $productVariantResolver;
    }

    public function createNew(): QuoteItemInterface
    {
        /** @var QuoteItemInterface $quoteItem */
        $quoteItem = $this->decoratedFactory->createNew();

        return $quoteItem;
    }

    public function createForProduct(ProductInterface $product): QuoteItemInterface
    {
        $quoteItem = $this->createNew();
        $variant = $this->productVariantResolver->getVariant($product);

        if (null !== $variant) {
            $quoteItem
                ->setVariant($variant)
                ->setVariantName($variant->getName())
            ;
        }

        $quoteItem->setProductName($product->getName());

        return $quoteItem;
    }
}
