<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Form\DataMapper;

use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Dedi\SyliusQuotePlugin\Modifier\QuoteItemQuantityModifierInterface;
use Symfony\Component\Form\Extension\Core\DataMapper\DataMapper;
use Symfony\Component\Form\FormInterface;
use Traversable;

class QuoteItemQuantityDataMapper implements QuoteItemQuantityDataMapperInterface
{
    private readonly DataMapper $dataMapper;

    private readonly QuoteItemQuantityModifierInterface $quoteItemQuantityModifier;

    public function __construct(
        DataMapper $dataMapper,
        QuoteItemQuantityModifierInterface $quoteItemQuantityModifier,
    ) {
        $this->dataMapper = $dataMapper;
        $this->quoteItemQuantityModifier = $quoteItemQuantityModifier;
    }

    public function mapDataToForms($viewData, $forms): void
    {
        $this->dataMapper->mapDataToForms($viewData, $forms);
    }

    /**
     * @param FormInterface[]|Traversable $forms
     * @param QuoteItemInterface $viewData
     */
    public function mapFormsToData($forms, &$viewData): void
    {
        $formsOtherThanQuantity = [];
        foreach ($forms as $form) {
            if ('quantity' === $form->getName()) {
                $targetQuantity = $form->getData();
                $this->quoteItemQuantityModifier->modify($viewData, (int) $targetQuantity);

                continue;
            }

            $formsOtherThanQuantity[] = $form;
        }

        if (!empty($formsOtherThanQuantity)) {
            $this->dataMapper->mapFormsToData($formsOtherThanQuantity, $viewData);
        }
    }
}
