<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Form\Type;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Sylius\Bundle\MoneyBundle\Form\Type\MoneyType;
use Sylius\Bundle\ResourceBundle\Form\Type\AbstractResourceType;
use Sylius\Bundle\ResourceBundle\Form\Type\ResourceAutocompleteChoiceType;
use Symfony\Component\Form\Extension\Core\Type\IntegerType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Constraints\Range;

class QuoteItemType extends AbstractResourceType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('variant', ResourceAutocompleteChoiceType::class, [
                'label' => false,
                'resource' => 'sylius.product_variant',
                'choice_name' => 'descriptor',
                'choice_value' => 'id',
                'attr' => [
                    'data-variant-field' => true,
                ],
                'constraints' => [
                    new NotBlank(['groups' => $this->validationGroups]),
                ],
            ])
            ->add('quantity', IntegerType::class, [
                'attr' => ['min' => 1],
                'label' => false,
                'empty_data' => 1,
                'constraints' => [
                    new Range(['min' => 1, 'groups' => $this->validationGroups]),
                ],
            ])
            ->add('unitPrice', MoneyType::class, [
                'currency' => $options['currency'],
                'label' => false,
                'constraints' => [
                    new NotBlank(['groups' => $this->validationGroups]),
                ],
            ])
        ;

        $builder->addEventListener(FormEvents::POST_SUBMIT, $this->setProductAndVariantName(...));
    }

    public function setProductAndVariantName(FormEvent $event): void
    {
        /** @var QuoteItemInterface $quoteItem */
        $quoteItem = $event->getData();

        if (null === ($variant = $quoteItem->getVariant())) {
            return;
        }

        $quoteItem
            ->setProductName($variant->getProduct()->getName())
            ->setVariantName($variant->getName() ?? '')
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        parent::configureOptions($resolver);

        $resolver
            ->setDefined(['currency'])
            ->setRequired('quote')
            ->setAllowedTypes('quote', [QuoteInterface::class])
        ;
    }

    public function getBlockPrefix(): string
    {
        return 'dedi_quote_item';
    }
}
